#pragma option -a-
#pragma option -k-
#pragma option -O1
#pragma option -w
#pragma option -w-asm
#pragma option -zC_TEXT
#pragma option -zR_DATA
#pragma option -zTDATA
#ifndef	__FLAT__
/*---------------------------------------------------------------------------
 * filename - delay.cas
 *
 * function(s)
 *        delay - wait for specified period.
 *--------------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*
 * filename - sound.cas
 *
 * function(s)
 *        sound   - outputs a sound of a specified frequency to the speaker
 *        nosound - turns off the speaker
 *-----------------------------------------------------------------------*/

/*
 *      C/C++ Run Time Library - Version 5.0
 *
 *      Copyright (c) 1987, 1992 by Borland International
 *      All Rights Reserved.
 *
 */


#pragma inline
#include <asmrules.h>
#include <dos.h>

static unsigned long multiplier = 1193*2L;

static void near dummy (void) {}

/*--------------------------------------------------------------------------*

Name            readtimer - read the complemented value of timer 0

Usage           unsigned readtimer (void);

Prototype in    local

Description     Obtain the complement of the value in timer 0.  The
                complement is used so that the timer will appear to
                count up rather than down.  The value returned will
                range from 0 to 0xffff.

Return value    The complement of the value in timer 0.

*---------------------------------------------------------------------------*/

static unsigned near readtimer (void)
{
  asm pushf                    /* Save interrupt flag                       */
  asm cli                      /* Disable interrupts                        */
  asm mov  al,0h               /* Latch timer 0                             */
  asm out  43h,al
      dummy();                 /* Waste some time */
  asm in   al,40h              /* Counter --> bx                            */
  asm mov  bl,al               /* LSB in BL                                 */
      dummy();                 /* Waste some time */
  asm in   al,40h
  asm mov  bh,al               /* MSB in BH                                 */
  asm not  bx                  /* Need ascending counter                    */
  asm popf                     /* Restore interrupt flag                    */
  return( _BX );
}

/*--------------------------------------------------------------------------*

Name            timer_init - initialize multiplier for delay function

Usage           void timer_init (void);

Prototype in    local

Description     Determine the multiplier required to convert milliseconds
                to an equivalent interval timer value.  Interval timer 0
		is normally programmed in mode 3 (square wave), where
                the timer is decremented by two every 840 nanoseconds;
                in this case the multiplier is 2386.  However, some
                programs and device drivers reprogram the timer in mode 2,
                where the timer is decremented by one every 840 ns; in this
                case the multiplier is halved, i.e. 1193.

                When the timer is in mode 3, it will never have an odd value.
                In mode 2, the timer can have both odd and even values.
                Therefore, if we read the timer 100 times and never
                see an odd value, it's a pretty safe assumption that
                it's in mode 3.  This is the method used in timer_init.

Return value    None

*---------------------------------------------------------------------------*/

static void timer_init(void)
{
    int i;

    for (i = 0; i < 100; i++)
        if ((readtimer() & 1) == 0)     /* readtimer() returns complement */
            {
            multiplier = 1193L;
            return;
            }
}

#pragma startup timer_init 16

/*--------------------------------------------------------------------------*

Name            delay - wait for specified period.

Usage           void delay(unsigned milliseconds);

Prototype in    dos.h

Description     The current thread of execution is suspended for the specified
		number of milliseconds.

Return value    None
*---------------------------------------------------------------------------*/

void _CType delay( unsigned milliseconds )
{
    unsigned long stop;
    unsigned cur, prev;

    stop = (prev = readtimer()) + (milliseconds * multiplier);

    while ((cur = readtimer()) < stop)
        {
        if (cur < prev)     /* Check for timer wraparound */
	    {
	    if (stop < 0x10000L)
		break;
	    stop -= 0x10000L;
	    }
	prev = cur;
	}
}

/*-----------------------------------------------------------------------*

Name            sound - outputs a sound of a specified frequency to the
			speaker

Usage           void sound(unsigned frequency)

Prototype in    dos.h

Description     outputs a sound of a specified frequency to the speaker

*------------------------------------------------------------------------*/
void sound(unsigned frequency)
{
asm     mov     bx,  frequency
asm     mov     ax,  34DDh
asm     mov     dx,  0012h
asm     cmp     dx,  bx
asm     jnb     stop
asm     div     bx
asm     mov     bx,  ax
asm     in      al,  61h
asm     test    al,  3
asm     jne     j1
asm     or      al,  3
asm     out     61h, al
asm     mov     al,  0B6h
asm     out     43h, al
j1:
asm     mov     al,  bl
asm     out     42h, al
asm     mov     al,  bh
asm     out     42h, al
stop: ;
}


/*-----------------------------------------------------------------------*

Name            nosound - turns off the speaker

Usage           void nosound(void)

Prototype in    dos.h

Description     turns the speaker off

*------------------------------------------------------------------------*/

void nosound(void)
/* Turns the speaker off */
{
asm     in      al,61H
asm     and     al, 0fcH
asm     out     61H, al

/*      outportb(0x61, inportb(0x61) & 0xfc); */
} /* nosound */
#endif/*__FLAT__*/
